;; @(#) drawrect.el -- Drawing rectangles in picture-mode
;;
;; Author:	    Jari Aalto <jari.aalto@poboxes.com>
;; Originally-by:   Howard Melman <melman@absolut.osf.org>
;; Keywords:	    tools
;;
;; Source was posted to Newsgroups gnu.emacs.sources in 03 Oct 1996
;; $Id: drawrect.el,v 1.3 1997/05/28 13:03:49 jaalto NotReleased $

;;; Installation:
;;
;; (autoload 'dr-rectangle "drawrect" t t)
;; (autoload 'dr-vaporize  "drawrect" t t)
;;
;; (add-hook 'picture-mode-hook 'my-picture-mode-hook)
;;
;; (defun my-picture-mode-hook  ()
;;   "My picture settings."
;;   (define-key picture-mode-map "\C-c\C-r" 'dr-rectangle)
;;   (define-key picture-mode-map "\C-c\C-v" 'dr-vaporize)
;;   )
;;

;;; Commentary:
;;
;; Draws a rectangle delimited by mark and point.
;;
;; I threw this together and it did what I needed it to.
;; At some point I'll expand this, but I figured others might be
;; interested before then.  Comments welcome.
;;

;;; History:

;;; Code:


(require 'picture)

(defvar dr-c ?\+
  "Corner character.")

(defvar dr-v ?\|
  "Vertical draw character.")

(defvar dr-h ?\-
  "Horizontal draw character.")

;; taken from the elisp manual
;; this really should be in the std distribution as about 5 copies of it
;; is used all over the place ...
;;
(eval-and-compile
  (if (not (fboundp 'current-line))
      (defun current-line ()
	"picture-rect.el.
       Return the vertical position of point.  Top line is 1."
	(+ (count-lines (point-min) (point))
	   (if (= (current-column) 0) 1 0)))))


(defun dr-insert (ch arg)
  "Insert CH in place of character previously at the cursor ARG times.
The cursor then moves in the direction you previously specified
with the commands `picture-movement-right', `picture-movement-up', etc.
Do \\[command-apropos] `picture-movement' to see those commands."
  (while (> arg 0)
    (setq arg (1- arg))
    (move-to-column (1+ (current-column)) t)
    (delete-char -1)
    (insert ch)
    (forward-char -1)
    (picture-move)))



(defun dr-rectangle (start end)
  "Draw a rectangle around region START END."
  (interactive "*r")                    ; start will be less than end
  (let* ((sl     (current-line))
         (sc     (current-column))
         (pvs    picture-vertical-step)
         (phs    picture-horizontal-step)
         (c1     (progn (goto-char start) (current-column)))
         (r1     (current-line))
         (c2     (progn (goto-char end) (current-column)))
         (r2     (current-line))
         (right  (max c1 c2))
         (left   (min c1 c2))
         (top    (min r1 r2))
         (bottom (max r1 r2)))
    (goto-line top)
    (move-to-column left)

    (picture-movement-right)
    (dr-insert dr-c 1)
    (dr-insert dr-h (- right (current-column)))

    (picture-movement-down)
    (dr-insert dr-c 1)
    (dr-insert dr-v (- bottom (current-line)))

    (picture-movement-left)
    (dr-insert dr-c 1)
    (dr-insert dr-h (- (current-column) left))

    (picture-movement-up)
    (dr-insert dr-c 1)
    (dr-insert dr-v (- (current-line) top))

    (picture-set-motion pvs phs)
    (goto-line sl)
    (move-to-column sc t)))




(defun dr-find-direction ()
  "Find direction where the line continues after dr-c character.

Return:
    Notice, this is also the order of evaluation: first one matched is
    returned.

     ---+       'left
     +---       'right


     |
     +          'up

     +          'down
     |"
  (let* ((pre   (preceding-char))
	 (next  (char-after (1+ (point))))
	 (line  (current-line))
	 (col   (current-column))
         )
    (cond
     ((eq pre dr-h)
      'left)
     ((eq next dr-h)
      'right)
     ((and (not (eq line 1))		;Can't go up in this line
	   (save-excursion
	     (forward-line -1)
	     (move-to-column col 'force)
	     (eq dr-v (char-after (point)))
	     ))
      'up)
     ((and (save-excursion		;Not last line?
	     (end-of-line)
	     (not (eobp)))
	   (save-excursion
	     (forward-line 1)
	     (move-to-column col 'force)
	     (eq dr-v (char-after (point)))
	     ))
      'down
      ))
    ))


(defun dr-kill-right (&optional left)
  "Kill all `dr-h' characters to the right, optionally to the LEFT.
Leave point after last killed character."
  (while (eq dr-h (char-after (point)))
    (insert " ")
    (forward-char 1)
    (delete-char -1)
    (if left
	(forward-char -2))
    ))

(defun dr-kill-down (&optional up)
  "Kill all `dr-v` characters down.,optionally UP.
Leave point after last killed character."
  (picture-movement-down)
  (while (eq dr-v (char-after (point)))
    (forward-char 1)
    (delete-char -1)
    (insert " ")
    (forward-char -1)
    (if up
        (picture-move-up 1)
      (picture-move-down 1))
    ))


(defun dr-vaporize  ()
  "Vaporize dr-v dr-c and dr-h character starting from current point.
You can use this function to kill drawn rectangle."
  (interactive)
  (let* ((ch       (char-after (point)))
	 (point    (point))
	 stat
	 pp col
	 )
    (if (not (memq ch (list dr-v dr-c dr-h)))
	(message "Nothing to do on current point.")

      (cond				;Until we find '+'
       ((eq ch dr-h)  (dr-kill-right))
       ((eq ch dr-v)  (dr-kill-down)))

      (while (eq dr-c (char-after (point)))
	(setq stat (dr-find-direction))
	(setq pp (point) col (current-column))
	(dr-insert ?\ 1)		;Delete '+'
	(goto-char pp)			;stay put

	(cond
	 ((eq stat 'right)
	  (forward-char 1)
	  (dr-kill-right))

	 ((eq stat 'left )
	  (forward-char -1)
	  (dr-kill-right 'left))

	 ((eq stat 'down )
	  (forward-line 1)
	  (move-to-column col)
    	  (dr-kill-down))

	 ((eq stat 'up)
	  (forward-line -1)
	  (move-to-column col)
	  (dr-kill-down 'up))
	 )

	;;  While end
	)
      (goto-char point)			;restore user position

      )))


(provide 'drawrect)

;;; drawrect.el ends here
